@@ -1,6 +1,6 @@
 /*
    This file is part of the KDE libraries
-   Copyright (c) 2005-2008 David Jarvie <djarvie@kde.org>
+   Copyright (c) 2005-2008,2011 David Jarvie <djarvie@kde.org>
    Copyright (c) 2005 S.R.Haque <srhaque@iee.org>.
 
    This library is free software; you can redistribute it and/or
@@ -19,12 +19,10 @@
    Boston, MA 02110-1301, USA.
 */
 
-// This file requires HAVE_STRUCT_TM_TM_ZONE to be defined if struct tm member tm_zone is available.
-// This file requires HAVE_TM_GMTOFF to be defined if struct tm member tm_gmtoff is available.
-
 #include "ktimezone.h"
 
 #include <config.h>
+#include <config-date.h> // SIZEOF_TIME_T
 
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -40,6 +38,7 @@
 #include <QtCore/QCoreApplication>
 
 #include <kdebug.h>
+#include <kglobal.h>
 
 int gmtoff(time_t t);   // defined in ksystemtimezone.cpp
 
@@ -118,10 +117,6 @@ void KTimeZones::clear()
 
 KTimeZone KTimeZones::zone(const QString &name) const
 {
-#ifdef Q_WS_WIN
-    // return always the utc for now
-    return KTimeZone::utc();
-#else
     if (!name.isEmpty())
     {
         ZoneMap::ConstIterator it = d->zones.constFind(name);
@@ -131,7 +126,6 @@ KTimeZone KTimeZones::zone(const QString &name) const
             return KTimeZone::utc();
     }
     return KTimeZone();    // error
-#endif
 }
 
 
@@ -249,7 +243,7 @@ KTimeZone::Transition::Transition()
 {
 }
 
-KTimeZone::Transition::Transition(const QDateTime &t, const KTimeZone::Phase &p) 
+KTimeZone::Transition::Transition(const QDateTime &t, const KTimeZone::Phase &p)
     : d(new KTimeZoneTransitionPrivate)
 {
     d->time  = t;
@@ -294,9 +288,9 @@ class KTimeZoneDataPrivate
         QList<KTimeZone::LeapSeconds> leapChanges;
         QList<int>                    utcOffsets;
         QList<QByteArray>             abbreviations;
-        int preUtcOffset;    // UTC offset to use before the first phase
+        KTimeZone::Phase              prePhase;    // phase to use before the first transition
 
-        KTimeZoneDataPrivate() : preUtcOffset(0) {}
+        KTimeZoneDataPrivate() {}
         // Find the last transition before a specified UTC or local date/time.
         int transitionIndex(const QDateTime &dt) const;
         bool transitionIndexes(const QDateTime &start, const QDateTime &end, int &ixstart, int &ixend) const;
@@ -309,7 +303,7 @@ class KTimeZoneDataPrivate
 class KTimeZonePrivate : public QSharedData
 {
 public:
-    KTimeZonePrivate() : source(0), data(0), refCount(1) {}
+    KTimeZonePrivate() : source(0), data(0), refCount(1), cachedTransitionIndex(-1) {}
     KTimeZonePrivate(KTimeZoneSource *src, const QString& nam,
                      const QString &country, float lat, float lon, const QString &cmnt);
     KTimeZonePrivate(const KTimeZonePrivate &);
@@ -325,7 +319,11 @@ public:
     float   latitude;
     float   longitude;
     mutable KTimeZoneData *data;
-    int refCount;
+    int     refCount; // holds the number of KTimeZoneBackend instances using the KTimeZonePrivate instance as a d-pointer.
+    int       cachedTransitionIndex;
+    QDateTime cachedTransitionStartZoneTime;
+    QDateTime cachedTransitionEndZoneTime;
+    bool      cachedTransitionTimesValid;
 
 private:
     static KTimeZoneSource *mUtcSource;
@@ -343,23 +341,29 @@ KTimeZonePrivate::KTimeZonePrivate(KTimeZoneSource *src, const QString& nam,
     latitude(lat),
     longitude(lon),
     data(0),
-    refCount(1)
+    refCount(1),
+    cachedTransitionIndex(-1)
 {
     // Detect duff values.
-    if ( latitude > 90 || latitude < -90 )
+    if (latitude > 90 || latitude < -90)
         latitude = KTimeZone::UNKNOWN;
-    if ( longitude > 180 || longitude < -180 )
+    if (longitude > 180 || longitude < -180)
         longitude = KTimeZone::UNKNOWN;
 }
 
 KTimeZonePrivate::KTimeZonePrivate(const KTimeZonePrivate &rhs)
-    : QSharedData(rhs),
-      source(rhs.source),
-      name(rhs.name),
-      countryCode(rhs.countryCode),
-      comment(rhs.comment),
-      latitude(rhs.latitude),
-      longitude(rhs.longitude)
+  : QSharedData(rhs),
+    source(rhs.source),
+    name(rhs.name),
+    countryCode(rhs.countryCode),
+    comment(rhs.comment),
+    latitude(rhs.latitude),
+    longitude(rhs.longitude),
+    refCount(1),
+    cachedTransitionIndex(rhs.cachedTransitionIndex),
+    cachedTransitionStartZoneTime(rhs.cachedTransitionStartZoneTime),
+    cachedTransitionEndZoneTime(rhs.cachedTransitionEndZoneTime),
+    cachedTransitionTimesValid(rhs.cachedTransitionTimesValid)
 {
     if (rhs.data)
         data = rhs.data->clone();
@@ -369,17 +373,24 @@ KTimeZonePrivate::KTimeZonePrivate(const KTimeZonePrivate &rhs)
 
 KTimeZonePrivate &KTimeZonePrivate::operator=(const KTimeZonePrivate &rhs)
 {
+    // Changing the contents of a KTimeZonePrivate instance by means of operator=() doesn't affect how
+    // many references to it are held.
     source      = rhs.source;
     name        = rhs.name;
     countryCode = rhs.countryCode;
     comment     = rhs.comment;
     latitude    = rhs.latitude;
     longitude   = rhs.longitude;
+    cachedTransitionIndex         = rhs.cachedTransitionIndex;
+    cachedTransitionStartZoneTime = rhs.cachedTransitionStartZoneTime;
+    cachedTransitionEndZoneTime   = rhs.cachedTransitionEndZoneTime;
+    cachedTransitionTimesValid    = rhs.cachedTransitionTimesValid;
     delete data;
     if (rhs.data)
         data = rhs.data->clone();
     else
         data = 0;
+    // refCount is unchanged
     return *this;
 }
 
@@ -401,9 +412,13 @@ void KTimeZonePrivate::cleanup()
 
 /******************************************************************************/
 
+K_GLOBAL_STATIC(KTimeZonePrivate, s_emptyTimeZonePrivate)
+
 KTimeZoneBackend::KTimeZoneBackend()
-  : d(new KTimeZonePrivate)
-{}
+  : d(&*s_emptyTimeZonePrivate)
+{
+    ++d->refCount;
+}
 
 KTimeZoneBackend::KTimeZoneBackend(const QString &name)
   : d(new KTimeZonePrivate(KTimeZonePrivate::utcSource(), name, QString(), KTimeZone::UNKNOWN, KTimeZone::UNKNOWN, QString()))
@@ -419,7 +434,7 @@ KTimeZoneBackend::KTimeZoneBackend(const KTimeZoneBackend &other)
 {
     ++d->refCount;
 }
-  
+
 KTimeZoneBackend::~KTimeZoneBackend()
 {
     if (d && --d->refCount == 0)
@@ -457,44 +472,91 @@ int KTimeZoneBackend::offsetAtZoneTime(const KTimeZone* caller, const QDateTime
             *secondOffset = 0;
         return 0;
     }
+    const QList<KTimeZone::Transition> transitions = caller->transitions();
+    int index = d->cachedTransitionIndex;
+    if (index >= 0 && index < transitions.count())
+    {
+        // There is a cached transition - check whether zoneDateTime uses it.
+        // Caching is used because this method has been found to consume
+        // significant CPU in real life applications.
+        if (!d->cachedTransitionTimesValid)
+        {
+            const int offset = transitions[index].phase().utcOffset();
+	    const int preoffset = (index > 0) ? transitions[index - 1].phase().utcOffset() : d->data ? d->data->previousUtcOffset() : 0;
+            d->cachedTransitionStartZoneTime = transitions[index].time().addSecs(qMax(offset, preoffset));
+            if (index + 1 < transitions.count())
+	    {
+                const int postoffset = transitions[index + 1].phase().utcOffset();
+                d->cachedTransitionEndZoneTime = transitions[index + 1].time().addSecs(qMin(offset, postoffset));
+	    }
+            d->cachedTransitionTimesValid = true;
+        }
+        QDateTime dtutc = zoneDateTime;
+        dtutc.setTimeSpec(Qt::UTC);
+        if (dtutc >= d->cachedTransitionStartZoneTime
+        &&  (index + 1 >= transitions.count() || dtutc < d->cachedTransitionEndZoneTime))
+        {
+            // The time falls within the cached transition limits, so return its UTC offset
+            const int offset = transitions[index].phase().utcOffset();
+            if (secondOffset)
+                *secondOffset = offset;
+#ifdef COMPILING_TESTS
+            kDebug(161) << "-> Using cache";   // enable the debug area to see this in the tests
+#endif
+            return offset;
+        }
+    }
+
+    // The time doesn't fall within the cached transition, or there isn't a cached transition
+#ifdef COMPILING_TESTS
+    kDebug(161) << "-> No cache";   // enable the debug area to see this in the tests
+#endif
     bool validTime;
+    int secondIndex = -1;
+    index = caller->transitionIndex(zoneDateTime, (secondOffset ? &secondIndex : 0), &validTime);
+    const KTimeZone::Transition* tr = (index >= 0) ? &transitions[index] : 0;
+    const int offset = tr ? tr->phase().utcOffset()
+                          : validTime ? (d->data ? d->data->previousUtcOffset() : 0)
+                                      : KTimeZone::InvalidOffset;
     if (secondOffset)
-    {
-        const KTimeZone::Transition *tr2;
-        const KTimeZone::Transition *tr = caller->transition(zoneDateTime, &tr2, &validTime);
-        if (!tr)
-        {
-            if (!validTime)
-                *secondOffset = KTimeZone::InvalidOffset;
-            else
-                *secondOffset = d->data ? d->data->previousUtcOffset() : 0;
-            return *secondOffset;
-        }
-        int offset = tr->phase().utcOffset();
-        *secondOffset = tr2 ? tr2->phase().utcOffset() : offset;
-        return offset;
-    }
-    else
-    {
-        const KTimeZone::Transition *tr = caller->transition(zoneDateTime, 0, &validTime);
-        if (!tr)
-        {
-            if (!validTime)
-                return KTimeZone::InvalidOffset;
-            return d->data ? d->data->previousUtcOffset() : 0;
-        }
-        return tr->phase().utcOffset();
-    }
+        *secondOffset = (secondIndex >= 0) ? transitions.at(secondIndex).phase().utcOffset() : offset;
+
+    // Cache transition data for subsequent date/time values which occur after the same transition.
+    d->cachedTransitionIndex = index;
+    d->cachedTransitionTimesValid = false;
+    return offset;
 }
 
 int KTimeZoneBackend::offsetAtUtc(const KTimeZone* caller, const QDateTime &utcDateTime) const
 {
     if (!utcDateTime.isValid()  ||  utcDateTime.timeSpec() != Qt::UTC)    // check for invalid time
         return 0;
-    const KTimeZone::Transition *tr = caller->transition(utcDateTime);
-    if (!tr)
-        return d->data ? d->data->previousUtcOffset() : 0;
-    return tr->phase().utcOffset();
+    const QList<KTimeZone::Transition> transitions = caller->transitions();
+    int index = d->cachedTransitionIndex;
+    if (index >= 0 && index < transitions.count())
+    {
+        // There is a cached transition - check whether utcDateTime uses it.
+        if (utcDateTime >= transitions[index].time()
+        &&  (index + 1 >= transitions.count()
+             || utcDateTime < transitions[index + 1].time()))
+        {
+            // The time falls within the cached transition, so return its UTC offset
+#ifdef COMPILING_TESTS
+            kDebug(161) << "Using cache";   // enable the debug area to see this in the tests
+#endif
+            return transitions[index].phase().utcOffset();
+        }
+    }
+
+    // The time doesn't fall within the cached transition, or there isn't a cached transition
+#ifdef COMPILING_TESTS
+    kDebug(161) << "No cache";   // enable the debug area to see this in the tests
+#endif
+    index = caller->transitionIndex(utcDateTime);
+    d->cachedTransitionIndex = index;   // cache transition data
+    d->cachedTransitionTimesValid = false;
+    const KTimeZone::Transition* tr = (index >= 0) ? &transitions.at(index) : 0;
+    return tr ? tr->phase().utcOffset() : (d->data ? d->data->previousUtcOffset() : 0);
 }
 
 int KTimeZoneBackend::offset(const KTimeZone* caller, time_t t) const
@@ -556,7 +618,7 @@ KTimeZone::KTimeZone(KTimeZoneBackend *impl)
   : d(impl)
 {
     // 'impl' should be a newly constructed object, with refCount = 1
-    Q_ASSERT(d->d->refCount == 1);
+    Q_ASSERT(d->d->refCount == 1 || d->d == &*s_emptyTimeZonePrivate);
 }
 
 KTimeZone &KTimeZone::operator=(const KTimeZone &tz)
@@ -650,7 +712,7 @@ QList<KTimeZone::Transition> KTimeZone::transitions(const QDateTime &start, cons
 }
 
 const KTimeZone::Transition *KTimeZone::transition(const QDateTime &dt, const Transition **secondTransition,
-                                                   bool *validTime ) const
+                                                   bool *validTime) const
 {
     if (!data(true))
         return 0;
@@ -696,8 +758,7 @@ void KTimeZone::setData(KTimeZoneData *data, KTimeZoneSource *source)
 {
     if (!isValid())
         return;
-    if (d->d->data)
-        delete d->d->data;
+    delete d->d->data;
     d->d->data = data;
     if (source)
         d->d->source = source;
@@ -730,7 +791,7 @@ QDateTime KTimeZone::toUtc(const QDateTime &zoneDateTime) const
 {
     if (!zoneDateTime.isValid()  ||  zoneDateTime.timeSpec() != Qt::LocalTime)
         return QDateTime();
-    int secs = offsetAtZoneTime(zoneDateTime);
+    const int secs = offsetAtZoneTime(zoneDateTime);
     if (secs == InvalidOffset)
         return QDateTime();
     QDateTime dt = zoneDateTime;
@@ -756,21 +817,22 @@ QDateTime KTimeZone::toZoneTime(const QDateTime &utcDateTime, bool *secondOccurr
             return dt;
         }
 
-        int index = d->d->data->transitionIndex(utcDateTime);
-        int secs = (index >= 0) ? d->d->data->transitions()[index].phase().utcOffset() : d->d->data->previousUtcOffset();
+        const KTimeZoneData *data = d->d->data;
+        const int index = data->transitionIndex(utcDateTime);
+        const int secs = (index >= 0) ? data->transitions().at(index).phase().utcOffset() : data->previousUtcOffset();
         QDateTime dt = utcDateTime.addSecs(secs);
         if (secondOccurrence)
         {
             // Check whether the local time occurs twice around a daylight savings time
             // shift, and if so, whether it's the first or second occurrence.
-            *secondOccurrence = d->d->data->d->isSecondOccurrence(dt, index);
+            *secondOccurrence = data->d->isSecondOccurrence(dt, index);
         }
         dt.setTimeSpec(Qt::LocalTime);
         return dt;
     }
     else
     {
-        int secs = offsetAtUtc(utcDateTime);
+        const int secs = offsetAtUtc(utcDateTime);
         QDateTime dt = utcDateTime.addSecs(secs);
         dt.setTimeSpec(Qt::LocalTime);
         if (secondOccurrence)
@@ -812,8 +874,8 @@ int KTimeZone::offset(time_t t) const
 int KTimeZone::currentOffset(Qt::TimeSpec basis) const
 {
     // Get current offset of this time zone to UTC
-    time_t now = time(0);
-    int secs = offset(now);
+    const time_t now = time(0);
+    const int secs = offset(now);
 
     switch (basis)
     {
@@ -848,15 +910,16 @@ KTimeZone KTimeZone::utc()
 
 QDateTime KTimeZone::fromTime_t(time_t t)
 {
-    static QDate epochDate(1970,1,1);
-    static QTime epochTime(0,0,0);
-    int days = t / 86400;
+    static const int secondsADay = 86400;
+    static const QDate epochDate(1970,1,1);
+    static const QTime epochTime(0,0,0);
+    int days = t / secondsADay;
     int secs;
     if (t >= 0)
-        secs = t % 86400;
+        secs = t % secondsADay;
     else
     {
-        secs = 86400 - (-t % 86400);
+        secs = secondsADay - (-t % secondsADay);
         --days;
     }
     return QDateTime(epochDate.addDays(days), epochTime.addSecs(secs), Qt::UTC);
@@ -864,14 +927,14 @@ QDateTime KTimeZone::fromTime_t(time_t t)
 
 time_t KTimeZone::toTime_t(const QDateTime &utcDateTime)
 {
-    static QDate epochDate(1970,1,1);
-    static QTime epochTime(0,0,0);
+    static const QDate epochDate(1970,1,1);
+    static const QTime epochTime(0,0,0);
     if (utcDateTime.timeSpec() != Qt::UTC)
         return InvalidTime_t;
-    qint64 days = epochDate.daysTo(utcDateTime.date());
-    qint64 secs = epochTime.secsTo(utcDateTime.time());
-    qint64 t64 = days * 86400 + secs;
-    time_t t = static_cast<time_t>(t64);
+    const qint64 days = epochDate.daysTo(utcDateTime.date());
+    const qint64 secs = epochTime.secsTo(utcDateTime.time());
+    const qint64 t64 = days * 86400 + secs;
+    const time_t t = static_cast<time_t>(t64);
     if (static_cast<qint64>(t) != t64)
         return InvalidTime_t;
     return t;
@@ -1015,7 +1078,7 @@ int KTimeZoneDataPrivate::transitionIndex(const QDateTime &dt) const
         dtutc.setTimeSpec(Qt::UTC);
         while (end - start > 1)
         {
-            int i = (start + end) / 2;
+            const int i = (start + end) / 2;
             if (dtutc.addSecs(-transitions[i].phase().utcOffset()) < transitions[i].time())
                 end = i;
             else
@@ -1060,13 +1123,13 @@ bool KTimeZoneDataPrivate::isSecondOccurrence(const QDateTime &utcLocalTime, int
 {
     if (transitionIndex < 0)
         return false;
-    int offset = transitions[transitionIndex].phase().utcOffset();
-    int prevoffset = (transitionIndex > 0) ? transitions[transitionIndex-1].phase().utcOffset() : preUtcOffset;
-    int phaseDiff = prevoffset - offset;
+    const int offset = transitions[transitionIndex].phase().utcOffset();
+    const int prevoffset = (transitionIndex > 0) ? transitions[transitionIndex-1].phase().utcOffset() : prePhase.utcOffset();
+    const int phaseDiff = prevoffset - offset;
     if (phaseDiff <= 0)
         return false;
     // Find how long after the start of the latest phase 'dt' is
-    int afterStart = transitions[transitionIndex].time().secsTo(utcLocalTime) - offset;
+    const int afterStart = transitions[transitionIndex].time().secsTo(utcLocalTime) - offset;
     return (afterStart < phaseDiff);
 }
 
@@ -1084,7 +1147,7 @@ KTimeZoneData::KTimeZoneData(const KTimeZoneData &c)
     d->leapChanges   = c.d->leapChanges;
     d->utcOffsets    = c.d->utcOffsets;
     d->abbreviations = c.d->abbreviations;
-    d->preUtcOffset  = c.d->preUtcOffset;
+    d->prePhase      = c.d->prePhase;
 }
 
 KTimeZoneData::~KTimeZoneData()
@@ -1099,7 +1162,7 @@ KTimeZoneData &KTimeZoneData::operator=(const KTimeZoneData &c)
     d->leapChanges   = c.d->leapChanges;
     d->utcOffsets    = c.d->utcOffsets;
     d->abbreviations = c.d->abbreviations;
-    d->preUtcOffset  = c.d->preUtcOffset;
+    d->prePhase      = c.d->prePhase;
     return *this;
 }
 
@@ -1130,9 +1193,8 @@ QByteArray KTimeZoneData::abbreviation(const QDateTime &utcDateTime) const
     if (d->phases.isEmpty())
         return "UTC";
     const KTimeZone::Transition *tr = transition(utcDateTime);
-    if (!tr)
-        return QByteArray();
-    const QList<QByteArray> abbrevs = tr->phase().abbreviations();
+    const QList<QByteArray> abbrevs = tr ? tr->phase().abbreviations()
+                                         : d->prePhase.abbreviations();
     if (abbrevs.isEmpty())
         return QByteArray();
     return abbrevs[0];
@@ -1144,7 +1206,7 @@ QList<int> KTimeZoneData::utcOffsets() const
     {
         for (int i = 0, end = d->phases.count();  i < end;  ++i)
         {
-            int offset = d->phases[i].utcOffset();
+            const int offset = d->phases[i].utcOffset();
             if (!d->utcOffsets.contains(offset))
                 d->utcOffsets.append(offset);
         }
@@ -1161,10 +1223,16 @@ QList<KTimeZone::Phase> KTimeZoneData::phases() const
     return d->phases;
 }
 
+void KTimeZoneData::setPhases(const QList<KTimeZone::Phase> &phases, const KTimeZone::Phase& previousPhase)
+{
+    d->phases   = phases;
+    d->prePhase = previousPhase;
+}
+
 void KTimeZoneData::setPhases(const QList<KTimeZone::Phase> &phases, int previousUtcOffset)
 {
-    d->phases = phases;
-    d->preUtcOffset = previousUtcOffset;
+    d->phases   = phases;
+    d->prePhase = KTimeZone::Phase(previousUtcOffset, QByteArray(), false);
 }
 
 bool KTimeZoneData::hasTransitions() const
@@ -1191,14 +1259,14 @@ void KTimeZoneData::setTransitions(const QList<KTimeZone::Transition> &transitio
 
 int KTimeZoneData::previousUtcOffset() const
 {
-    return d->preUtcOffset;
+    return d->prePhase.utcOffset();
 }
 
 const KTimeZone::Transition *KTimeZoneData::transition(const QDateTime &dt, const KTimeZone::Transition **secondTransition,
                                                        bool *validTime) const
 {
     int secondIndex;
-    int index = transitionIndex(dt, (secondTransition ? &secondIndex : 0), validTime);
+    const int index = transitionIndex(dt, (secondTransition ? &secondIndex : 0), validTime);
     if (secondTransition)
         *secondTransition = (secondIndex >= 0) ? &d->transitions[secondIndex] : 0;
     return (index >= 0) ? &d->transitions[index] : 0;
@@ -1225,13 +1293,13 @@ int KTimeZoneData::transitionIndex(const QDateTime &dt, int *secondIndex, bool *
          */
         QDateTime dtutc = dt;
         dtutc.setTimeSpec(Qt::UTC);
-        int count = d->transitions.count();
-        int next = (index >= 0) ? index + 1 : 0;
+        const int count = d->transitions.count();
+        const int next = (index >= 0) ? index + 1 : 0;
         if (next < count)
         {
             KTimeZone::Phase nextPhase = d->transitions[next].phase();
-            int offset = (index >= 0) ? d->transitions[index].phase().utcOffset() : d->preUtcOffset;
-            int phaseDiff = nextPhase.utcOffset() - offset;
+            const int offset = (index >= 0) ? d->transitions[index].phase().utcOffset() : d->prePhase.utcOffset();
+            const int phaseDiff = nextPhase.utcOffset() - offset;
             if (phaseDiff > 0)
             {
                 // Get UTC equivalent as if 'dt' was in the next phase
