@@ -31,6 +31,7 @@
 #include "xml/dom_nodelistimpl.h"
 #include "xml/dom_textimpl.h"
 #include "xml/dom2_traversalimpl.h"
+#include "xml/security_origin.h"
 #include "misc/shared.h"
 #include "misc/loader.h"
 #include "misc/seed.h"
@@ -559,8 +560,13 @@ public:
     // Returns 0 if this is the top level document.
     HTMLPartContainerElementImpl *ownerElement() const;
 
+    khtml::SecurityOrigin* origin() const;
+    void setOrigin(khtml::SecurityOrigin*);
+    
+    // These represent JS operations on domain strings, rather than full-blown origins.
+    // (so no port, protocol, etc.)
+    void setDomain( const DOMString &newDomain ); 
     DOMString domain() const;
-    void setDomain( const DOMString &newDomain ); // not part of the DOM
 
     bool isURLAllowed(const QString& url) const;
 
@@ -598,8 +604,8 @@ public:
 
     JSEditor *jsEditor();
 
-    QHash<QString,khtml::CounterNode*>* counters(const khtml::RenderObject* o) { return m_counterDict.value(o); }
-    void setCounters(const khtml::RenderObject* o, QHash<QString,khtml::CounterNode*> *dict) { m_counterDict.insert(o, dict);}
+    QHash<DOMString,khtml::CounterNode*>* counters(const khtml::RenderObject* o) { return m_counterDict.value(o); }
+    void setCounters(const khtml::RenderObject* o, QHash<DOMString,khtml::CounterNode*> *dict) { m_counterDict.insert(o, dict);}
     void removeCounters(const khtml::RenderObject* o) { delete m_counterDict.take(o); }
 
     ElementMappingCache& underDocNamedCache() {
@@ -681,7 +687,7 @@ protected:
     QList<NodeImpl*> m_maintainsState;
 
     // ### evaluate for placement in RenderStyle
-    QHash<const khtml::RenderObject*,QHash<QString,khtml::CounterNode*> *> m_counterDict;
+    QHash<const khtml::RenderObject*,QHash<DOMString,khtml::CounterNode*> *> m_counterDict;
 
     khtml::DynamicDomRestyler *m_dynamicDomRestyler;
 
@@ -721,7 +727,8 @@ protected:
     SharedPtr<khtml::RenderArena> m_renderArena;
 private:
     JSEditor *m_jsEditor;
-    mutable DOMString m_domain;
+    mutable RefPtr<khtml::SecurityOrigin> m_origin;
+
     int m_selfOnlyRefCount;
 public:
     // Nodes belonging to this document hold "self-only" references -
@@ -762,7 +769,6 @@ class DocumentFragmentImpl : public NodeBaseImpl
 {
 public:
     DocumentFragmentImpl(DocumentImpl *doc);
-    DocumentFragmentImpl(const DocumentFragmentImpl &other);
 
     // DOM methods overridden from  parent classes
     virtual DOMString nodeName() const;
