@@ -157,56 +157,110 @@ ThemePackage::ThemePackage(QObject *parent)
     : Plasma::PackageStructure(parent, QString("Plasma Theme"))
 {
     addDirectoryDefinition("dialogs", "dialogs/", i18n("Images for dialogs"));
+
     addFileDefinition("dialogs/background", "dialogs/background.svg",
                       i18n("Generic dialog background"));
+    addFileDefinition("dialogs/background", "dialogs/background.svgz",
+                      i18n("Generic dialog background"));
+
     addFileDefinition("dialogs/shutdowndialog", "dialogs/shutdowndialog.svg",
                       i18n("Theme for the logout dialog"));
+    addFileDefinition("dialogs/shutdowndialog", "dialogs/shutdowndialog.svgz",
+                      i18n("Theme for the logout dialog"));
 
     addDirectoryDefinition("wallpapers", "wallpapers/", i18n("Wallpaper packages"));
     addDirectoryDefinition("animations", "animations/", i18n("Animation scripts"));
 
     addDirectoryDefinition("widgets", "widgets/", i18n("Images for widgets"));
+
     addFileDefinition("widgets/background", "widgets/background.svg",
                       i18n("Background image for widgets"));
+    addFileDefinition("widgets/background", "widgets/background.svgz",
+                      i18n("Background image for widgets"));
+
     addFileDefinition("widgets/clock", "widgets/clock.svg",
                       i18n("Analog clock face"));
+    addFileDefinition("widgets/clock", "widgets/clock.svgz",
+                      i18n("Analog clock face"));
+
     addFileDefinition("widgets/panel-background", "widgets/panel-background.svg",
                       i18n("Background image for panels"));
+    addFileDefinition("widgets/panel-background", "widgets/panel-background.svgz",
+                      i18n("Background image for panels"));
+
     addFileDefinition("widgets/plot-background", "widgets/plot-background.svg",
                       i18n("Background for graphing widgets"));
+    addFileDefinition("widgets/plot-background", "widgets/plot-background.svg",
+                      i18n("Background for graphing widgets"));
+
     addFileDefinition("widgets/tooltip", "widgets/tooltip.svg",
                       i18n("Background image for tooltips"));
+    addFileDefinition("widgets/tooltip", "widgets/tooltip.svgz",
+                      i18n("Background image for tooltips"));
 
     addDirectoryDefinition("opaque/dialogs", "opaque/dialogs/", i18n("Opaque images for dialogs"));
+
     addFileDefinition("opaque/dialogs/background", "opaque/dialogs/background.svg",
                       i18n("Opaque generic dialog background"));
+    addFileDefinition("opaque/dialogs/background", "opaque/dialogs/background.svgz",
+                      i18n("Opaque generic dialog background"));
+
     addFileDefinition("opaque/dialogs/shutdowndialog", "opaque/dialogs/shutdowndialog.svg",
                       i18n("Opaque theme for the logout dialog"));
+    addFileDefinition("opaque/dialogs/shutdowndialog", "opaque/dialogs/shutdowndialog.svgz",
+                      i18n("Opaque theme for the logout dialog"));
 
     addDirectoryDefinition("opaque/widgets", "opaque/widgets/", i18n("Opaque images for widgets"));
+
     addFileDefinition("opaque/widgets/panel-background", "opaque/widgets/panel-background.svg",
                       i18n("Opaque background image for panels"));
+    addFileDefinition("opaque/widgets/panel-background", "opaque/widgets/panel-background.svgz",
+                      i18n("Opaque background image for panels"));
+
     addFileDefinition("opaque/widgets/tooltip", "opaque/widgets/tooltip.svg",
                       i18n("Opaque background image for tooltips"));
+    addFileDefinition("opaque/widgets/tooltip", "opaque/widgets/tooltip.svgz",
+                      i18n("Opaque background image for tooltips"));
 
     addDirectoryDefinition("locolor/dialogs", "locolor/dialogs/",
                            i18n("Low color images for dialogs"));
+
     addFileDefinition("locolor/dialogs/background", "locolor/dialogs/background.svg",
                       i18n("Low color generic dialog background"));
+    addFileDefinition("locolor/dialogs/background", "locolor/dialogs/background.svgz",
+                      i18n("Low color generic dialog background"));
+
     addFileDefinition("locolor/dialogs/shutdowndialog", "locolor/dialogs/shutdowndialog.svg",
                       i18n("Low color theme for the logout dialog"));
+    addFileDefinition("locolor/dialogs/shutdowndialog", "locolor/dialogs/shutdowndialog.svgz",
+                      i18n("Low color theme for the logout dialog"));
 
     addDirectoryDefinition("locolor/widgets", "locolor/widgets/", i18n("Images for widgets"));
+
     addFileDefinition("locolor/widgets/background", "locolor/widgets/background.svg",
                       i18n("Low color background image for widgets"));
+    addFileDefinition("locolor/widgets/background", "locolor/widgets/background.svgz",
+                      i18n("Low color background image for widgets"));
+
     addFileDefinition("locolor/widgets/clock", "locolor/widgets/clock.svg",
                       i18n("Low color analog clock face"));
+    addFileDefinition("locolor/widgets/clock", "locolor/widgets/clock.svgz",
+                      i18n("Low color analog clock face"));
+
     addFileDefinition("locolor/widgets/panel-background", "locolor/widgets/panel-background.svg",
                       i18n("Low color background image for panels"));
+    addFileDefinition("locolor/widgets/panel-background", "locolor/widgets/panel-background.svgz",
+                      i18n("Low color background image for panels"));
+
     addFileDefinition("locolor/widgets/plot-background", "locolor/widgets/plot-background.svg",
                       i18n("Low color background for graphing widgets"));
+    addFileDefinition("locolor/widgets/plot-background", "locolor/widgets/plot-background.svgz",
+                      i18n("Low color background for graphing widgets"));
+
     addFileDefinition("locolor/widgets/tooltip", "locolor/widgets/tooltip.svg",
                       i18n("Low color background image for tooltips"));
+    addFileDefinition("locolor/widgets/tooltip", "locolor/widgets/tooltip.svgz",
+                      i18n("Low color background image for tooltips"));
 
     addFileDefinition("colors", "colors", i18n("KColorScheme configuration file"));
 
